@@ -30,7 +30,7 @@ class Request(RequestBase):
             return self.__req.connection.server.server_hostname # mod_python 2
 
     def get_header(self, name):
-        if self.__req.headers_in.has_key(name):
+        if name in self.__req.headers_in:
             return self.__req.headers_in[name]
 
     def write_header(self, name, value):
