@@ -28,12 +28,12 @@ class Redirect:
         self.loc = loc
 
 class ResponseMixin:
-
-    '''Maintain an ordered dictionary of headers.
+    '''
+    Maintain an ordered, case-insensitive dictionary of headers.
     '''
 
     def __init__(self):
-        self.__sent_headers = 0
+        self.__sent_headers = False
         self.__seqnum = 0
         self.__headers = {}
         self.set_header('Pragma', 'no-cache')
@@ -100,7 +100,7 @@ class ResponseMixin:
             for value in values:
                 self.request.write_header(name, value)
         self.request.end_headers()
-        self.__sent_headers = 1
+        self.__sent_headers = True
 
     def send_content(self, data):
         if not self.__sent_headers:
@@ -195,7 +195,7 @@ class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):
         self.__enter_page(name, args)
 
     def pop_page(self, target_page = None):
-        while 1:
+        while True:
             try:
                 name = self.locals.__pages__.pop()
             except IndexError:
@@ -212,14 +212,14 @@ class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):
     def req_equals(self, name):
         try:
             if self.request.field_value(name):
-                return 1
+                return True
         except KeyError:
             try:
                 if self.request.field_value(name + '.x'):
-                    return 1
+                    return True
             except KeyError:
                 pass
-        return 0
+        return False
 
     def base_url(self):
         return self.app.base_url()
@@ -248,7 +248,7 @@ class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):
         if loc.startswith('/'):
             return urlparse.urlunparse((scheme, netloc, loc, '', '', ''))
         new_base = self.absolute_base_url()
-        if new_base[-1] != '/':
+        if not new_base.endswith('/'):
             new_base = new_base + '/'
         return urlparse.urlunparse((scheme, netloc, new_base + loc, '', '', ''))
 
@@ -296,16 +296,15 @@ class Application(ResourceMixin):
         import linecache
         etype, value, tb = sys.exc_info()
         try:
-            list = traceback.format_exception(etype, value, tb)
-            pyexc = ''.join(list)
-            list = []
-            list.append('Template traceback (most recent call last):')
+            pyexc = ''.join(traceback.format_exception(etype, value, tb))
+            lines = []
+            lines.append('Template traceback (most recent call last):')
             for tup in self.template_traceback(tb):
-                list.append('  File "%s", line %s, in %s' % tup)
+                lines.append('  File "%s", line %s, in %s' % tup)
                 line = linecache.getline(tup[0], tup[1]).rstrip()
                 if line:
-                    list.append(line)
-            htmlexc = '\n'.join(list)
+                    lines.append(line)
+            htmlexc = '\n'.join(lines)
         finally:
             del etype, value, tb
         return pyexc, htmlexc
@@ -333,19 +332,19 @@ class Application(ResourceMixin):
             self.remove_session(ctx)
 
     def template_traceback(self, tb):
-        list = []
+        tmpl_frames = []
         prev = None
         while tb is not None:
             f = tb.tb_frame
-            if f.f_locals.has_key('self'):
+            if 'self' in f.f_locals:
                 obj = f.f_locals['self']
                 if hasattr(obj, 'line_num'):
                     tag = (obj.filename, obj.line_num, obj.name)
                     if tag != prev:
-                        list.append(tag)
+                        tmpl_frames.append(tag)
                         prev = tag
             tb = tb.tb_next
-        return list
+        return tmpl_frames
 
     def load_session(self, ctx):
         ctx.load_session()
@@ -357,7 +356,7 @@ class Application(ResourceMixin):
         ctx.remove_session()
 
     def validate_request(self, ctx):
-        return 1
+        return True
 
     def base_url(self):
         return self.__base_url
@@ -449,23 +448,23 @@ class PageModuleMixin:
         return module
 
     def page_enter(self, ctx, args):
-        if hasattr(ctx.page, 'page_enter'):
-            func = getattr(ctx.page, 'page_enter')
+        func = getattr(ctx.page, 'page_enter', None)
+        if func is not None:
             func(ctx, *args)
 
     def page_leave(self, ctx):
-        if hasattr(ctx.page, 'page_leave'):
-            func = getattr(ctx.page, 'page_leave')
+        func = getattr(ctx.page, 'page_leave', None)
+        if func is not None:
             func(ctx)
 
     def process_request(self, ctx):
-        if hasattr(ctx.page, 'page_process'):
-            func = getattr(ctx.page, 'page_process')
+        func = getattr(ctx.page, 'page_process', None)
+        if func is not None:
             func(ctx)
 
     def display_response(self, ctx):
-        if hasattr(ctx.page, 'page_display'):
-            func = getattr(ctx.page, 'page_display')
+        func = getattr(ctx.page, 'page_display', None)
+        if func is not None:
             func(ctx)
 
 
@@ -493,26 +492,26 @@ class PageObjectMixin:
 
     def page_enter(self, ctx, args):
         obj = self.__page_objects[ctx.page]
-        if hasattr(obj, 'page_enter'):
-            func = getattr(obj, 'page_enter')
+        func = getattr(obj, 'page_enter', None)
+        if func is not None:
             func(ctx, *args)
 
     def page_leave(self, ctx):
         obj = self.__page_objects[ctx.page]
-        if hasattr(obj, 'page_leave'):
-            func = getattr(obj, 'page_leave')
+        func = getattr(obj, 'page_leave', None)
+        if func is not None:
             func(ctx)
 
     def process_request(self, ctx):
         obj = self.__page_objects[ctx.page]
-        if hasattr(obj, 'page_process'):
-            func = getattr(obj, 'page_process')
+        func = getattr(obj, 'page_process', None)
+        if func is not None:
             func(ctx)
 
     def display_response(self, ctx):
         obj = self.__page_objects[ctx.page]
-        if hasattr(obj, 'page_display'):
-            func = getattr(obj, 'page_display')
+        func = getattr(obj, 'page_display', None)
+        if func is not None:
             func(ctx)
 
 # ------------------------------------------------------------------
@@ -577,12 +576,14 @@ class SimpleSessionApp(PickleSignMixin,
                        SessionServerAppMixin):
 
     def __init__(self, base_url, template_path, start_page, secret,
-                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
+                 session_appid, session_server = 'localhost', 
+                 server_port = 34343, session_age = 1800):
         Application.__init__(self, base_url)
         PickleSignMixin.__init__(self, secret)
         CachingTemplateLoaderMixin.__init__(self, template_path)
         PageObjectMixin.__init__(self, start_page)
-        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)
+        SessionServerAppMixin.__init__(self, session_appid, session_server, 
+                                       server_port, session_age)
 
     def create_context(self):
         return SessionAppContext(self)
@@ -612,12 +613,14 @@ class ModularSessionApp(PickleSignMixin,
                         SessionServerAppMixin):
 
     def __init__(self, base_url, module_path, template_path, start_page, secret,
-                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
+                 session_appid, session_server = 'localhost', 
+                 server_port = 34343, session_age = 1800):
         Application.__init__(self, base_url)
         PickleSignMixin.__init__(self, secret)
         CachingTemplateLoaderMixin.__init__(self, template_path)
         PageModuleMixin.__init__(self, module_path, start_page)
-        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)
+        SessionServerAppMixin.__init__(self, session_appid, session_server, 
+                                       server_port, session_age)
 
     def create_context(self):
         return SessionAppContext(self)
