@@ -17,21 +17,24 @@ class Request(RequestBase):
             fields = cgi.FieldStorage()
         RequestBase.__init__(self, fields)
 
+    def get_param(self, name, default=None):
+        return os.environ.get(name, default)
+
     def get_uri(self):
-        return os.environ.get('REQUEST_URI')
+        return self.get_param('REQUEST_URI')
 
     def get_method(self):
-        return os.environ.get('REQUEST_METHOD')
+        return self.get_param('REQUEST_METHOD')
 
     def get_path_info(self):
-        return os.environ.get('PATH_INFO')
+        return self.get_param('PATH_INFO')
 
     def get_servername(self):
-        return os.environ.get('HTTP_HOST')
+        return self.get_param('HTTP_HOST')
 
     def get_header(self, name):
         env_name = 'HTTP_' + name.upper().replace('-', '_')
-        return os.environ.get(env_name)
+        return self.get_param(env_name)
 
     def write_header(self, name, value):
         sys.stdout.write('%s: %s\n' % (name, value))
