@@ -7,7 +7,6 @@
 # to run
 import os
 import sys
-import stat
 import re
 import cPickle
 import __builtin__
@@ -18,7 +17,7 @@ try:
 except ImportError:
     have_zlib = 0
 import base64
-import md5
+import hmac, sha
 
 from albatross.template import Template, check_tagname
 from albatross.common import *
@@ -120,20 +119,42 @@ class ExecuteMixin:
     def __init__(self):
         self.reset_content()
 
-    def get_macro_arg(self, name):
-        for i in range(len(self.__macro_stack) - 1, -1, -1):
-            args = self.__macro_stack[i]
-            if args.has_key(name):
-                return args[name]
-        raise ApplicationError('undefined macro argument "%s"' % name)
+    def reset_content(self):
+        self.__macro_stack = [{}]
+        self.__active_select = None
+        self.__trap_stack = []
+        self.__content_parts = []
 
-    def push_macro_args(self, args):
-        self.__macro_stack.append(args)
+    def get_macro_arg(self, name):
+        try:
+            return self.__macro_stack[-1][name]
+        except KeyError:
+            raise ApplicationError('undefined macro argument "%s"' % name)
+
+    def push_macro_args(self, args, defaults=None):
+        if defaults is not None:
+            ns = defaults.copy()
+            ns.update(self.__macro_stack[-1])
+        else:
+            ns = self.__macro_stack[-1].copy()
+        ns.update(args)
+        self.__macro_stack.append(ns)
 
     def pop_macro_args(self):
-        args = self.__macro_stack[-1]
-        del self.__macro_stack[-1]
-        return args
+        return self.__macro_stack.pop(-1)
+
+    def set_active_select(self, select, value):
+        if self.__active_select is not None:
+            raise ApplicationError('Can not nest <al-select>')
+        self.__active_select = select, value
+
+    def clear_active_select(self):
+        self.__active_select = None
+
+    def get_active_select(self):
+        if self.__active_select is None:
+            raise ApplicationError('<al-option> must be within an <al-select>')
+        return self.__active_select
 
     def push_content_trap(self):
         self.__trap_stack.append(self.__content_parts)
@@ -146,7 +167,7 @@ class ExecuteMixin:
         return data
 
     def write_content(self, data):
-        if type(data) is type(u''):
+        if isinstance(data, unicode):
             data = data.encode('utf-8')
         self.__content_parts.append(data)
 
@@ -163,11 +184,6 @@ class ExecuteMixin:
         sys.stdout.write(data)
         sys.stdout.flush()
 
-    def reset_content(self):
-        self.__macro_stack = []
-        self.__trap_stack = []
-        self.__content_parts = []
-
 # ------------------------------------------------------------------
 # Template file loaders
 # ------------------------------------------------------------------
@@ -183,7 +199,7 @@ class TemplateLoaderMixin:
         self.__loaded_names = {}
 
     def load_template(self, name):
-        self.__loaded_names[name] = 1
+        self.__loaded_names[name] = True
         path = os.path.join(self.__base_dir, name)
         try:
             text = open(path).read()
@@ -193,7 +209,7 @@ class TemplateLoaderMixin:
         return Template(self, path, text)
 
     def load_template_once(self, name):
-        if self.__loaded_names.get(name):
+        if name in self.__loaded_names:
             return
         return self.load_template(name)
 
@@ -211,7 +227,7 @@ class CachingTemplateLoaderMixin:
     def load_template(self, name):
         path = os.path.join(self.__base_dir, name)
         try:
-            mtime = os.stat(path)[stat.ST_MTIME]
+            mtime = os.path.getmtime(path)
         except OSError, e:
             raise TemplateLoadError("%s: %s" % (path, e.strerror))
         templ = self.__cache.get(path)
@@ -275,20 +291,30 @@ class Point:
 
 class NameRecorderMixin:
 
-    NORMAL, LIST, FILE = range(3)
+    NORMAL = 0
+    MULTI = 1
+    MULTISINGLE = 2
+    FILE = 3
+
+    modes = {
+        NORMAL: 'single input returning single value',
+        MULTI: 'multiple inputs returning a list',
+        MULTISINGLE: 'multiple inputs returning single value',
+        FILE: 'file input',
+    }
 
     def __init__(self):
         self.__elem_names = {}
 
     def form_open(self):
         self.__elem_names = {}
-        self.__need_multipart_enc = 0
-        self.__needs_close = 1
+        self.__need_multipart_enc = False
+        self.__needs_close = True
 
     def form_close(self):
         if not self.__needs_close:
             raise ApplicationError('<al-form> elements must not be nested')
-        self.__needs_close = 0
+        self.__needs_close = False
         text = cPickle.dumps(self.__elem_names, -1)
         text = self.app.pickle_sign(text)
         if have_zlib:
@@ -302,63 +328,60 @@ class NameRecorderMixin:
 
     def input_add(self, itype, name, unused_value = None, return_list = 0):
         if itype == 'file':
-            self.__need_multipart_enc = 1
-            self.__elem_names[name] = self.FILE
-        elif self.__elem_names.has_key(name):
-            prev_multiples = self.__elem_names[name]
-            implicit_multi = itype in ('radio', 'submit', 'image')
-            if not return_list and not implicit_multi:
-                raise ApplicationError(
-                    'al-input "%s" not defined as "list"' % name)
-            if return_list and implicit_multi:
-                raise ApplicationError(
-                    'al-input "%s" should not be defined as "list"' % name)
-            if prev_multiples == 0 and return_list != 0:
-                raise ApplicationError(
-                    'al-input "%s" initially not defined as "list"' % name)
-            if return_list == 0 and prev_multiples != 0:
-                raise ApplicationError(
-                    'al-input "%s" initially defined as "list"' % name)
-        else:
+            self.__need_multipart_enc = True
+            mode = self.FILE
+        elif itype in ('radio', 'submit', 'image'):
             if return_list:
-                self.__elem_names[name] = self.LIST
-            else:
-                self.__elem_names[name] = self.NORMAL
+                raise ApplicationError('%s input "%s" should not be defined '
+                                       'as "list"' % (itype, name))
+            mode = self.MULTISINGLE
+        elif return_list:
+            mode = self.MULTI
+        else:
+            mode = self.NORMAL
+        if name in self.__elem_names:
+            prev_mode = self.__elem_names[name]
+            if prev_mode != mode:
+                raise ApplicationError('%s input "%s" was "%s", now defined '
+                                       'as "%s"' % (itype, name, 
+                                                    self.modes.get(prev_mode), 
+                                                    self.modes.get(mode)))
+            elif mode == self.NORMAL:
+                raise ApplicationError('input "%s" appears more than once, '
+                                       'but is not defined as "list"' % name)
+        else:
+            self.__elem_names[name] = mode
 
     def merge_request(self):
-        if not self.request.has_field('__albform__'):
-            for name in self.request.field_names():
-                value = self.request.field_value(name)
-                self.set_value(name, value)
-            return
-        text = self.request.field_value('__albform__')
-        text = base64.decodestring(text)
-        if have_zlib:
-            text = zlib.decompress(text)
-        text = self.app.pickle_unsign(text)
-        if not text:
-            return
-        elem_names = cPickle.loads(text)
-        for name, mode in elem_names.items():
-            if mode == self.FILE:
-                value = self.request.field_file(name)
-            elif self.request.has_field(name):
-                value = self.request.field_value(name)
-            else:
-                x_name = '%s.x' % name
-                y_name = '%s.y' % name
-                if self.request.has_field(x_name) \
-                   and self.request.has_field(y_name):
-                    value = Point(int(self.request.field_value(x_name)),
-                                  int(self.request.field_value(y_name)))
+        if self.request.has_field('__albform__'):
+            text = self.request.field_value('__albform__')
+            text = base64.decodestring(text)
+            if have_zlib:
+                text = zlib.decompress(text)
+            text = self.app.pickle_unsign(text)
+            if not text:
+                return
+            elem_names = cPickle.loads(text)
+            for name, mode in elem_names.items():
+                if mode == self.FILE:
+                    value = self.request.field_file(name)
+                elif self.request.has_field(name):
+                    value = self.request.field_value(name)
                 else:
-                    value = None
-            if mode == self.LIST:
-                if not value:
-                    value = []
-                elif type(value) is not type([]):
-                    value = [value]
-            self.set_value(name, value)
+                    x_name = '%s.x' % name
+                    y_name = '%s.y' % name
+                    if self.request.has_field(x_name) \
+                    and self.request.has_field(y_name):
+                        value = Point(int(self.request.field_value(x_name)),
+                                    int(self.request.field_value(y_name)))
+                    else:
+                        value = None
+                if mode == self.MULTI:
+                    if not value:
+                        value = []
+                    elif not isinstance(value, list):
+                        value = [value]
+                self.set_value(name, value)
 
 # ------------------------------------------------------------------
 # Handle execution context local namespace - for expression eval().
@@ -391,7 +414,7 @@ class NamespaceMixin:
             del self.locals.__ctx__
 
     def set_value(self, name, value):
-        if name[0] == '_':
+        if name.startswith('_'):
             raise SecurityError('cannot merge %s into namespace' % name)
         # handle iterator back door; op,iter[,value]
         elems = name.split(',')
@@ -471,6 +494,18 @@ class NamespaceMixin:
             finally:
                 del exc_tb
 
+    def merge_vars(self, *vars):
+        """
+        Copy named fields from the request object to the local
+        namespace.  Does intelligent prefix matching, so 'foo.bar'
+        matches 'foo.bar[23]' or 'foo.bar.baz', but not 'foo.bargin'.
+        """
+        for name in self.request.field_names():
+            for var in vars:
+                if (name == var 
+                        or (name.startswith(var) and name[len(var)] in '.[')):
+                    self.set_value(name, self.request.field_value(name))
+
     def _get_value(self, name):
         # handle iterator back door; op,iter[,value]
         elems = name.split(',')
@@ -502,15 +537,15 @@ class NamespaceMixin:
     def has_value(self, name):
         try:
             self._get_value(name)
-            return 1
+            return True
         except (AttributeError, IndexError, NameError):
-            return 0
+            return False
 
     def has_values(self, *names):
         for name in names:
             if not self.has_value(name):
-                return 0
-        return 1
+                return False
+        return True
 
 
 # ------------------------------------------------------------------
@@ -523,21 +558,19 @@ class PickleSignMixin:
         self.__secret = secret
 
     def pickle_sign(self, text):
-        m = md5.new()
-        m.update(self.__secret)
+        m = hmac.new(self.__secret, digestmod=sha)
         m.update(text)
         text = m.digest() + text
         return text
 
     def pickle_unsign(self, text):
-        digest = text[:16]
-        text = text[16:]
-        m = md5.new()
-        m.update(self.__secret)
+        m = hmac.new(self.__secret, digestmod=sha)
+        digest = text[:m.digest_size]
+        text = text[m.digest_size:]
         m.update(text)
         if m.digest() == digest:
             return text
-        return ''
+        raise SecurityError
 
 
 # ------------------------------------------------------------------
@@ -576,35 +609,35 @@ class SessionBase:
 
     def __init__(self):
         self.__init_vars()
-        self.__save_session = 1
+        self.__save_session = True
 
     def add_session_vars(self, *names):
-        if len(names) == 1 and type(names[0]) in (type([]), type(())):
+        if len(names) == 1 and isinstance(names[0], (list, tuple)):
             names = names[0]
         for name in names:
-            assert isinstance(name, type(""))
+            assert isinstance(name, str)
             if not hasattr(self.locals, name):
                 raise ApplicationError('add "%s" to locals first' % name)
         for name in names:
-            self.__vars[name] = 1
+            self.__vars[name] = True
 
     def default_session_var(self, name, value):
         if not hasattr(self.locals, name):
             setattr(self.locals, name, value)
-        self.add_session_vars(name)
+        self.__vars[name] = True
 
     def del_session_vars(self, *names):
-        if len(names) == 1 and type(names[0]) in (type([]), type(())):
+        if len(names) == 1 and isinstance(names[0], (list, tuple)):
             names = names[0]
         for name in names:
-            if self.__vars.has_key(name):
+            if name in self.__vars:
                 del self.__vars[name]
 
     def session_vars(self):
         return self.__vars.keys()
 
     def __init_vars(self):
-        self.__vars = {'__page__': 1, '__pages__': 1}
+        self.__vars = {'__page__': True, '__pages__': True}
 
     def remove_session(self):
         self.__init_vars()
@@ -626,7 +659,7 @@ class SessionBase:
                 raise ApplicationError("can't unpickle session")
             self.locals.__dict__.update(vars)
             for name in vars.keys():
-                self.__vars[name] = 1
+                self.__vars[name] = True
         finally:
             __builtin__.__import__ = real_imp
 
