@@ -23,24 +23,24 @@ class RequestFields:
         self.__fields = fields
 
     def has_field(self, name):
-        return self.__fields.has_key(name)
+        return name in self.__fields
 
     def field_value(self, name):
         field = self.__fields[name]
-        if isinstance(field, type([])):
-            if isinstance(field[0], type('')):
+        if isinstance(field, list):
+            if isinstance(field[0], str):
                 # For checkboxes, this is a vec of strings in mod_python 2
                 return field
             # But a vec of utils.FieldStorage objects in mod_python 3
             return [f.value for f in field]
-        elif isinstance(field, type('')):
+        elif isinstance(field, str):
             return field                # Weird - blame mod_apache?
         else:
             return field.value
 
     def field_file(self, name):
         field = self.__fields[name]
-        if isinstance(field, type([])):
+        if isinstance(field, list):
             return [FileField(f) for f in field]
         if field.type.startswith('multipart'):
             return [FileField(f) for f in field.value]
