@@ -70,10 +70,10 @@ class SessionFileContextMixin(SessionCookieMixin, SessionBase):
 
 class SessionFileAppMixin:
 
-    def __init__(self, appid, session_dir, age = 1800):
-        self.__appid = appid
+    def __init__(self, session_appid, session_dir, session_age = 1800):
+        self.__appid = session_appid
         self.__session_dir = session_dir
-        self.__age = age
+        self.__age = session_age
 
     def ses_appid(self):
         return self.__appid
