@@ -45,7 +45,7 @@ class SessionDict:
         return ses
 
     def set_session(self, sesid, text):
-        if not self.dict.has_key(sesid):
+        if sesid not in self.dict:
             return
         ses = self.dict[sesid]
         self.usage = self.usage - len(ses.text) + len(text)
@@ -59,12 +59,12 @@ class SessionDict:
         return ses
 
     def del_session(self, sesid):
-        if self.dict.has_key(sesid):
+        if sesid in self.dict:
             self.usage = self.usage = len(self.dict[sesid].text)
             del self.dict[sesid]
 
     def has_session(self, sesid):
-        return self.dict.has_key(sesid)
+        return sesid in self.dict
 
 
 class SesLife:
@@ -320,10 +320,10 @@ class Client:
             else:
                 self.write('ERROR unrecognised command: %s\r\n' % str)
             if self.error_count > 0:
-                self.error_count = self.error_count - 1
+                self.error_count -= 1
             return 1
         except:
-            self.error_count = self.error_count + 1
+            self.error_count += 1
             if self.error_count > MAX_CLIENT_EXCEPTIONS:
                 return 0
             traceback.print_exc()
