@@ -6,7 +6,7 @@
 #
 import time
 
-from albatross.template import Content, EmptyTag, EnclosingTag, \
+from albatross.template import Content, EmptyTag, EnclosingTag, Text, \
                                template_version, template_features
 from albatross.common import *
 
@@ -33,10 +33,8 @@ class InputNameValueMixin:
         alias = self.get_attrib('alias')
         if alias:
             name = ctx.make_alias(alias)
-        elif self.has_attrib('nameexpr'):
-            name = self.eval_attrib(ctx, 'nameexpr')
         else:
-            name = self.get_attrib('name')
+            name = self.get_expr_attrib(ctx, 'name')
         if not name:
             self.raise_error('"name" attribute is null')
         return name
@@ -52,9 +50,7 @@ class InputNameValueMixin:
             return name, ctx.get_value(name)
 
     def input_add(self, ctx, itype, name, value, multiple):
-        if (not (self.has_attrib('disabledbool') 
-                 and self.eval_attrib(ctx, 'disabledbool')) 
-            and not self.has_attrib('disabled')):
+        if not self.get_bool_attrib(ctx, 'disabled'):
             ctx.input_add(itype, name, value, multiple)
 
 # ------------------------------------------------------------------
@@ -77,9 +73,7 @@ class Input(EmptyTag, InputNameValueMixin):
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
         self.__itype = self.get_attrib('type', 'text').lower()
-        try:
-            self.unbound_to_html = self.type_dict[self.__itype]
-        except KeyError:
+        if self.__itype not in self.type_dict:
             self.raise_error('unrecognised <INPUT TYPE="%s">' % self.__itype)
         self.assert_any_attrib(*self.naming_attrs)
 
@@ -121,7 +115,8 @@ class Input(EmptyTag, InputNameValueMixin):
 
     def to_html(self, ctx):
         ctx.write_content('<input')
-        self.unbound_to_html(self, ctx)
+        unbound_to_html = self.type_dict[self.__itype]
+        unbound_to_html(self, ctx)
         ctx.write_content(' />')
 
     def generic_to_html(self, ctx):
@@ -152,7 +147,7 @@ class Input(EmptyTag, InputNameValueMixin):
             value = 'on'
         ctx.write_content(' name="%s"' % name)
         ctx.write_content(' value="%s"' % value_attr)
-        if type(value) in (type([]), type(())):
+        if isinstance(value, (list, tuple)):
             if str(value_attr) in map(str, value):
                 ctx.write_content(' checked')
         elif str(value) and str(value) == str(value_attr):
@@ -212,9 +207,8 @@ class Href(EnclosingTag, InputNameValueMixin):
                 href = self.eval_attrib(ctx, 'expr')
             else:
                 href = self.get_attrib('href')
-            pos = href.find('?')
-            if pos < 0:
-                if href.find('=') >= 0:
+            if '?' not in href:
+                if '=' in href:
                     # make into request on current page
                     href = '%s?%s' % (ctx.current_url(), href)
                 else:
@@ -279,13 +273,13 @@ class Form(EnclosingTag):
 # options will need to use <al-option> tags in template files.  The
 # alternative is to define the option list in Python code.
 #
-# Two forms implemented:
-# <al-option>12</al-option>
-# <al-option value="12">Arbitrary text</al-option>
+# HTML 4.01 defines the <option> tag's behaviour in the following way:
+#  - if a /value/ attribute is given, this is the control's value, otherwise
+#    the value is derived from the contained content.
+#  - if a /label/ attribute is given, this forms the control's visible label,
+#    otherwise the visible label is derived from the contained content.
 #
-# In the first form, we need to evaluate the content of the tag before
-# we output the start tag so we can determine whether or not to set
-# the selected attribute on the option, hence the use of the content_trap.
+# We use the content_trap to capture the contained content, if need be.
 class Option(EnclosingTag):
 
     name = 'al-option'
@@ -293,26 +287,38 @@ class Option(EnclosingTag):
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
 
-    def to_html(self, ctx, select_value):
-        if self.has_attrib('value'):
-            value = self.get_attrib('value')
+    def to_html(self, ctx):
+        select, select_value = ctx.get_active_select()
+        value = self.get_expr_attrib(ctx, 'value')
+        if isinstance(value, (tuple, list)):
+            value, label = value
         else:
+            label = self.get_expr_attrib(ctx, 'label')
+        if not value or not label:
             ctx.push_content_trap()
             EnclosingTag.to_html(self, ctx)
-            value = ctx.pop_content_trap()
+            contained = ctx.pop_content_trap()
+        value = value or contained
+        label = label or contained or value
         ctx.write_content('<option')
-        self.write_attribs_except(ctx)
-        if type(select_value) is type(''):
-            if value == select_value:
-                ctx.write_content(' selected')
-        elif type(select_value) in (type([]), type(())):
-            if value in select_value:
-                ctx.write_content(' selected')
+        self.write_attribs_except(ctx, 'value', 'valueexpr', 
+                                       'label', 'labelexpr',
+                                       'noescape', 'selectedbool')
+        if value != label:
+            ctx.write_content(' value="%s"' % escape(value))
+        if self.has_attrib('selected') or self.has_attrib('selectedbool'):
+            selected = self.get_bool_attrib(ctx, 'selected')
+        elif isinstance(select_value, str):
+            selected = (value == select_value)
+        elif isinstance(select_value, (list, tuple)):
+            selected = (value in select_value)
+        if selected:
+            ctx.write_content(' selected')
         ctx.write_content('>')
-        if self.has_attrib('value'):
-            EnclosingTag.to_html(self, ctx)
+        if self.has_attrib('noescape'):
+            ctx.write_content(label)
         else:
-            ctx.write_content(value)
+            ctx.write_content(escape(label))
         ctx.write_content('</option>')
 
 
@@ -329,7 +335,6 @@ class Select(EnclosingTag, InputNameValueMixin):
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
         self.assert_any_attrib('name', 'alias', 'nameexpr')
-        self.options = []
 
     def to_html(self, ctx):
         name, value = self.get_name_and_value(ctx)
@@ -337,7 +342,7 @@ class Select(EnclosingTag, InputNameValueMixin):
         self.write_attribs_except(ctx, 'expr', 'optionexpr', 'alias', 'name',
                                   'nameexpr', 'noescape', 'list')
         ctx.write_content(' name="%s">' % name)
-        if type(value) in (type([]), type(())):
+        if isinstance(value, (list, tuple)):
             value = map(str, value)
         else:
             value = str(value)
@@ -346,26 +351,27 @@ class Select(EnclosingTag, InputNameValueMixin):
             for item in seq:
                 self.option_to_html(ctx, item, value)
         else:
-            for item in self.options:
-                item.to_html(ctx, value)
+            ctx.set_active_select(self, value)
+            EnclosingTag.to_html(self, ctx)
+            ctx.clear_active_select()
         ctx.write_content('</select>')
         self.input_add(ctx, 'select', name, None, 
                        self.has_attrib('list') or self.has_attrib('multiple'))
 
     def option_to_html(self, ctx, item, select_value):
-        if type(item) is type(()):
+        if isinstance(item, (list, tuple)):
             value, text = map(str, item)
         else:
             value = str(item)
             text = value
         ctx.write_content('<option')
-        if type(select_value) is type(''):
+        if isinstance(select_value, str):
             if value == select_value:
                 ctx.write_content(' selected')
-        elif type(select_value) in (type([]), type(())):
+        elif isinstance(select_value, (list, tuple)):
             if value in select_value:
                 ctx.write_content(' selected')
-        if type(item) is type(()):
+        if isinstance(item, (list, tuple)):
             ctx.write_content(' value="%s"' % escape(value))
         ctx.write_content('>')
         if self.has_attrib('noescape'):
@@ -375,12 +381,6 @@ class Select(EnclosingTag, InputNameValueMixin):
         ctx.write_content('</option>')
         ctx.write_content('\n')
 
-    def append(self, tag):
-        if isinstance(tag, Option):
-            self.options.append(tag)
-        else:
-            EnclosingTag.append(self, tag)
-
 
 class TextArea(EnclosingTag, InputNameValueMixin):
 
@@ -630,12 +630,69 @@ class ListIterator:
         self._have_value = 1
 
     def next(self):
-        self._index = self._index + 1
-        self._count = self._count + 1
+        self._index += 1
+        self._count += 1
         self._have_value = None
         self.has_value()
 
 
+class ListIteratorLite:
+    """
+    A simpler ListIterator for cases were we don't need pagination, etc
+    This one does not index the sequence, using only the iterator protocol.
+    """
+
+    def __init__(self):
+        self._index = -1
+        self._iter = self._have_value = None
+
+    def has_sequence(self):
+        return self._iter is not None
+
+    def set_sequence(self, seq):
+        self._iter = iter(seq)
+
+    # ----------------------------------------------------
+    # Iteration.  Make sure that each element is only retrieved from
+    # the sequence once.  This allows programs to make use of objects
+    # which are not really sequences; popen().readline() for instance.
+
+    def reset_index(self):
+        pass
+
+    def reset_count(self):
+        pass
+
+    def pagesize(self):
+        return 0
+
+    def index(self):
+        return self._index
+
+    def value(self):
+        return self._value
+
+    def clear_value(self):
+        self._have_value = None
+
+    def has_value(self):
+        if not self._have_value:
+            try:
+                self._value = self._iter.next()
+                self._have_value = True
+                self._index += 1
+            except StopIteration:
+                self._have_value = False
+        return self._have_value
+
+    def set_value(self, value):
+        self._value = value
+        self._have_value = True
+
+    def next(self):
+        self._have_value = False
+
+
 # EXPR:seq
 #   Yields an object which implements sequence protocol.  The result
 #   is assigned to the ITER attribute.  If not present, the ITER
@@ -662,21 +719,28 @@ class For(EnclosingTag, InputNameValueMixin):
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_any_attrib('iter')
+        self.assert_any_attrib('iter', 'expr')
         if self.has_attrib('flow'):
             if self.get_attrib('flow').lower() not in ('across', 'down'):
                 self.raise_error('"flow" must be "across" or "down"')
 
     def to_html(self, ctx):
-        # If PAGESIZE or CONTINUE defined then the iterator will not
-        # be recreated if it already exists.
         name = self.get_attrib('iter')
         iter = None
+        # If PAGESIZE or CONTINUE defined then the iterator will not
+        # be recreated if it already exists.
         if self.has_attrib('pagesize') or self.has_attrib('continue'):
+            if not name:
+                raise ApplicationError('iterator must be named if "pagesize" '
+                                       'or "continue" are used')
             iter = ctx.get_value(name)
         if iter is None:
-            iter = ListIterator()
-            ctx.set_value(name, iter)
+            if self.has_attrib('pagesize') or self.has_attrib('cols'):
+                iter = ListIterator()
+            else:
+                iter = ListIteratorLite()
+            if name:
+                ctx.set_value(name, iter)
         # If iterator does not have a sequence yet, the EXPR attribute
         # must be present
         if self.has_attrib('expr'):
@@ -684,13 +748,17 @@ class For(EnclosingTag, InputNameValueMixin):
             iter.set_sequence(seq)
         elif not iter.has_sequence():
             self.raise_error('no sequence present; expected "expr" attribute')
+        # Place iterator value into namespace?
+        var_names = self.get_attrib('vars')
+        if var_names:
+            var_names = commasplit(var_names)
         # If PAGESIZE is defined then place the iterator into the
         # session.
         if self.has_attrib('pagesize'):
             ctx.add_session_vars(name)
             pagesize = int(self.get_attrib('pagesize'))
             iter.set_pagesize(pagesize)
-        # If CONTINUE if defined then the contents will flow on from
+        # If CONTINUE is defined then the contents will flow on from
         # the previous use of the iterator
         if not self.has_attrib('continue'):
             iter.reset_index()
@@ -710,6 +778,13 @@ class For(EnclosingTag, InputNameValueMixin):
             while iter.has_value():
                 if pagesize and iter.count() >= pagesize:
                     break
+                if var_names:
+                    value = iter.value()
+                    if len(var_names) == 1:
+                        ctx.set_value(var_names[0], value)
+                    else:
+                        for a, v in zip(var_names, value):
+                            ctx.set_value(a, v)
                 EnclosingTag.to_html(self, ctx)
                 iter.next()
             iter.clear_value()
@@ -828,7 +903,7 @@ class LazyTreeIterator(TreeIterator):
     def is_open(self):
         if not hasattr(self._value, 'children'):
             return 0
-        return self._open_aliases.has_key(self._key)
+        return self._key in self._open_aliases
 
     def load_children(self, ctx, node):
         if not node.children_loaded:
@@ -838,13 +913,13 @@ class LazyTreeIterator(TreeIterator):
     def node_is_open(self, ctx, node):
         if not hasattr(node, 'children'):
             return 0
-        node_open = self._open_aliases.has_key(node.albatross_alias())
+        node_open = node.albatross_alias() in self._open_aliases
         if node_open:
             self.load_children(ctx, node)
         return node_open
 
     def is_selected(self):
-        return self._selected_aliases.has_key(self._key)
+        return self._key in self._selected_aliases
 
     def node_type(self):
         return self._type
@@ -863,7 +938,7 @@ class LazyTreeIterator(TreeIterator):
     def select_alias(self, alias, value = 1):
         if value:
             self._selected_aliases[alias] = 1
-        elif self._selected_aliases.has_key(alias):
+        elif alias in self._selected_aliases:
             del self._selected_aliases[alias]
 
     def set_selected_aliases(self, aliases):
@@ -879,7 +954,7 @@ class LazyTreeIterator(TreeIterator):
     def open_alias(self, alias, open = 1):
         if open:
             self._open_aliases[alias] = 1
-        elif self._open_aliases.has_key(alias):
+        elif alias in self._open_aliases:
             del self._open_aliases[alias]
 
     def set_open_aliases(self, aliases):
@@ -889,7 +964,7 @@ class LazyTreeIterator(TreeIterator):
 
     def get_backdoor(self, op, key):
         if op == 'treeselect':
-            return self._selected_aliases.has_key(key)
+            return key in self._selected_aliases
         elif op == 'treefold':
             return 1
         else:
@@ -902,11 +977,11 @@ class LazyTreeIterator(TreeIterator):
                     self._selected_aliases = { key: 1 }
                 else:
                     self._selected_aliases[key] = 1
-            elif self._selected_aliases.has_key(key):
+            elif key in self._selected_aliases:
                 del self._selected_aliases[key]
         elif op == 'treefold':
             if value:
-                if self._open_aliases.has_key(key):
+                if key in self._open_aliases:
                     del self._open_aliases[key]
                 else:
                     self._open_aliases[key] = 1
@@ -1107,10 +1182,10 @@ class Lookup(EnclosingTag):
 
     def _build_item_dict(self, ctx):
         if not hasattr(self, 'item_dict'):
-            dict = {}
+            item_dict = {}
             for item in self.item_list:
-                dict[item.eval_attrib(ctx, 'expr')] = item
-            self.item_dict = dict
+                item_dict[item.eval_attrib(ctx, 'expr')] = item
+            self.item_dict = item_dict
 
     def to_html(self, ctx):
         self._build_item_dict(ctx)
@@ -1128,6 +1203,15 @@ class Lookup(EnclosingTag):
 #
 # Usearg and Macro do not produce output when evaluated - only when
 # expanded via <al-expand>
+class Setdefault(EnclosingTag):
+
+    name = 'al-setdefault'
+
+    def __init__(self, ctx, filename, line_num, attribs):
+        EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
+        self.assert_has_attrib('name')
+
+
 class Usearg(EmptyTag):
 
     name = 'al-usearg'
@@ -1149,8 +1233,15 @@ class Macro(EnclosingTag):
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
         self.assert_has_attrib('name')
+        self.arg_defaults = {}
         ctx.register_macro(self.get_attrib('name'), self)
 
+    def append(self, tag):
+        if isinstance(tag, Setdefault):
+            self.arg_defaults[tag.get_attrib('name')] = tag
+        else:
+            EnclosingTag.append(self, tag)
+
     def to_html(self, ctx):
         pass
 
@@ -1163,13 +1254,30 @@ class Setarg(EnclosingTag):
     name = 'al-setarg'
 
 
+class _MacroArgExpr:
+    def __init__(self, expr):
+        self.expr = expr
+        self.co_expr = None
+
+    def to_html(self, ctx):
+        if self.co_expr is None:
+            self.co_expr = compile(self.expr, '<albatross>', 'eval')
+        ctx.write_content(ctx.eval_expr(self.co_expr))
+        
+
 class Expand(EnclosingTag):
 
     name = 'al-expand'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.arg_dict = {}
+        self.arg_dict = {None: self.content}
+        self.arg_expr = {}
+        for name, value in self.attrib_items():
+            if name.endswith('arg'):
+                self.arg_dict[name[:-3]] = Text(value)
+            elif name.endswith('argexpr'):
+                self.arg_dict[name[:-7]] = _MacroArgExpr(value)
 
     def append(self, tag):
         if isinstance(tag, Setarg):
@@ -1181,14 +1289,11 @@ class Expand(EnclosingTag):
         macro = ctx.get_macro(self.get_attrib('name'))
         if not macro:
             self.raise_error('undefined macro "%s"' % self.get_attrib('name'))
-        args = {}
-        args[None] = self.content
-        for name, value in self.arg_dict.items():
-            args[name] = value
-        ctx.push_macro_args(args)
+        ctx.push_macro_args(self.arg_dict, macro.arg_defaults)
         macro.expand_html(ctx)
         ctx.pop_macro_args()
 
+
 class Require(EmptyTag):
 
     name = 'al-require'
@@ -1214,4 +1319,5 @@ class Require(EmptyTag):
 
 tags = (Input, Href, Img, Form, Option, Select, TextArea,
         Flush, Comment, Include, Elif, Else, If, Exec, For, Tree,
-        Value, Item, Lookup, Usearg, Macro, Setarg, Expand, Require)
+        Value, Item, Lookup, Setdefault, Usearg, Macro, Setarg, Expand, 
+        Require)
