@@ -82,10 +82,10 @@ class Tag:
     def has_attrib(self, name):
         '''Was the named attribute named with the tag
         '''
-        return self.attribs.has_key(name)
+        return name in self.attribs
 
     def assert_has_attrib(self, name):
-        if not self.attribs.has_key(name):
+        if name not in self.attribs:
             self.raise_error('missing "%s" attribute' % name)
 
     def assert_any_attrib(self, *attrs):
@@ -111,6 +111,27 @@ class Tag:
         '''
         return self.attribs.get(name, default)
 
+    def get_expr_attrib(self, ctx, name, default=None):
+        """
+        If /name/expr version of attribute exists, fetch it, evaluate
+        and return the result, otherwise fetch the value of /name/.
+        """
+        if name + 'expr' in self.attribs:
+            return self.eval_attrib(ctx, name + 'expr')
+        else:
+            return self.attribs.get(name, default)
+
+    def get_bool_attrib(self, ctx, name, default=False):
+        """
+        If /name/bool version of attribute exists, fetch it, evaluate
+        it in a boolean context and return the result, otherwise return
+        whether /name/ exists.
+        """
+        if name + 'bool' in self.attribs:
+            return bool(self.eval_attrib(ctx, name + 'bool'))
+        else:
+            return name in self.attribs
+
     def set_attrib_order(self, order):
         self.attrib_order = order
 
@@ -132,7 +153,7 @@ class Tag:
                 if self.eval_attrib(ctx, name):
                     ctx.write_content(' %s' % name[:-len('bool')])
             else:
-                if name.endswith('expr'):
+                if name[:-4] and name.endswith('expr'):
                     value = self.eval_attrib(ctx, name)
                     name = name[:-len('expr')]
                 else:
@@ -356,18 +377,18 @@ class Template:
         line_num = 1
         for part in _re_tag.split(text):
             if state == TAG_TEXT:
-                str = part
-                if str:
+                text = part
+                if text:
                     if white_space == WHITE_STRIP:
-                        if str[0] == '\n':
-                            str = _re_leading.sub('', str)
+                        if text.startswith('\n'):
+                            text = _re_leading.sub('', text)
                     elif white_space == WHITE_INDENT:
-                        if str[0] == '\n':
-                            str = str[1:]
+                        if text.startswith('\n'):
+                            text = text[1:]
                     elif white_space == WHITE_NEWLINE:
-                        str = '\n'
-                if str:
-                    parent.append(Text(str))
+                        text = '\n'
+                if text:
+                    parent.append(Text(text))
                 white_space = WHITE_STRIP
                 state = TAG_NAME
             elif state == TAG_NAME:
@@ -396,7 +417,7 @@ class Template:
                 state = TAG_ATTRIBS
             elif state == TAG_ATTRIBS:
                 bits = _re_attrib.split(part)
-                dict = {}
+                attribs = {}
                 attrib_order = []
                 offset = 0
                 while 1:
@@ -421,13 +442,13 @@ class Template:
                                                    (filename, line_num, value))
                         white_space = value
                     else:
-                        dict[name] = value
+                        attribs[name] = value
                         attrib_order.append(name)
                 if is_open:
                     if tagclass is AnyTag:
-                        tag = tagclass(ctx, filename, line_num, tagname, dict)
+                        tag = tagclass(ctx, filename, line_num, tagname, attribs)
                     else:
-                        tag = tagclass(ctx, filename, line_num, dict)
+                        tag = tagclass(ctx, filename, line_num, attribs)
                     tag.set_attrib_order(attrib_order)
                     parent.append(tag)
                     if tag.has_content():
@@ -442,7 +463,7 @@ class Template:
                 else:
                     pass
                 state = TAG_TEXT
-            line_num = line_num + part.count('\n')
+            line_num += part.count('\n')
         if state not in (TAG_TEXT, TAG_NAME):
             raise ApplicationError('%s:%s: incomplete tag definition' % \
                                    (filename, line_num))
