@@ -37,7 +37,12 @@ class TreePage:
         ctx.run_template('tree.html')
 
 
-app = SimpleSessionApp('tree.py', '.', 'start', 'tree', '-=-secret-=-')
+app = SimpleSessionApp(base_url='tree.py',
+                       template_path='.',
+                       start_page='start',
+                       secret='-=-secret-=-',
+                       session_appid='tree')
+
 app.register_page('start', TreePage())
 
 
