@@ -6,7 +6,7 @@
 # 
 # Form functionality tests.
 #
-# $Id: pagemodule.py 6167 2004-01-15 05:31:02Z andrewm $
+# $Id: pagemodule.py 8841 2007-03-12 03:13:16Z andrewm $
 
 import sys
 import os
@@ -14,33 +14,7 @@ import unittest
 import albatross
 import re
 
-class DummyRequest:
-    def __init__(self):
-        self.__content = []
-
-    def has_field(self, name):
-        pass
-
-    def field_names(self):
-        return []
-
-    def write_header(self, name, value):
-        pass
-
-    def end_headers(self):
-        pass
-
-    def write_content(self, data):
-        self.__content.append(data)
-
-    def get_content(self):
-        return '\n'.join(self.__content)
-
-    def set_status(self, status):
-        pass
-
-    def return_code(self):
-        pass
+from dummyrequest import DummyRequest
 
 class TestContext(albatross.AppContext,
                   albatross.NameRecorderMixin,
