@@ -9,7 +9,7 @@
 # Unfortunately, we need to have intimate knowledge of the encoding scheme used
 # by the NameRecorderMixin - changes there will need to be reflected here.
 #
-# $Id: recorder.py 6214 2005-05-19 14:04:28Z andrewm $
+# $Id: recorder.py 8887 2007-03-16 09:35:33Z andrewm $
 
 import re, os, sys
 
@@ -101,7 +101,7 @@ class MergeTestRequest:
         self.expect['__pages__'] = []
 
     def has_field(self, field):
-        return self.fields.has_key(field)
+        return field in self.fields
 
     def field_value(self, field):
         return self.fields[field]
@@ -117,6 +117,15 @@ class MergeTestCase(unittest.TestCase):
 """
 Test namespace "protection"
 """
+class UnderscoreTestContext(albatross.AppContext, albatross.StubRecorderMixin,
+                          albatross.StubSessionMixin):
+    def __init__(self, template_dir):
+        app = RecorderTestApp(template_dir)
+        albatross.AppContext.__init__(self, app)
+
+#    def current_url(self):
+#        return 'here'
+
 class UnderscoreRequest:
     def __init__(self):
         self.__field = '_protected'
@@ -135,7 +144,7 @@ class UnderscoreRequest:
 
 class UnderscoreTestCase(unittest.TestCase):
     def runTest(self):
-        ctx = RecorderTestContext('.')
+        ctx = UnderscoreTestContext('.')
         ctx.set_request(UnderscoreRequest())
         self.failUnlessRaises(albatross.SecurityError, ctx.merge_request)
 
