@@ -5,7 +5,7 @@
 #
 # Test the NamespaceMixin's set_value method
 #
-# $Id: set_value.py 6245 2005-12-16 05:19:32Z andrewm $
+# $Id: set_value.py 8849 2007-03-13 04:49:27Z andrewm $
 
 import unittest
 import albatross
@@ -29,22 +29,22 @@ class SetValueTest(unittest.TestCase):
 
     def errors_test(self):
         self.locals.b = 1
-        self.assertRaises(AttributeError, self._test, 'a.b', 1)
-        self.assertRaises(ValueError, self._test, 'b[a]', 1)
-        self.assertRaises(SyntaxError, self._test, 'b[.a]', 1)
-        self.assertRaises(SyntaxError, self._test, 'b[1.0]', 1)
-        self.assertRaises(SyntaxError, self._test, 'b[1', 1)
-        self.assertRaises(SyntaxError, self._test, 'b]', 1)
-        self.assertRaises(SyntaxError, self._test, '.b', 1)
-        self.assertRaises(SyntaxError, self._test, 'b.', 1)
-        self.assertRaises(SyntaxError, self._test, '[0]', 1)
+        self.assertRaises(AttributeError, set_value, self, 'a.b', 1)
+        self.assertRaises(ValueError, set_value, self, 'b[a]', 1)
+        self.assertRaises(SyntaxError, set_value, self, 'b[.a]', 1)
+        self.assertRaises(SyntaxError, set_value, self, 'b[1.0]', 1)
+        self.assertRaises(SyntaxError, set_value, self, 'b[1', 1)
+        self.assertRaises(SyntaxError, set_value, self, 'b]', 1)
+        self.assertRaises(SyntaxError, set_value, self, '.b', 1)
+        self.assertRaises(SyntaxError, set_value, self, 'b.', 1)
+        self.assertRaises(SyntaxError, set_value, self, '[0]', 1)
 
     def set_array_test(self):
         self.locals.b = [0] * 4
         self._test('b[0]', 1)
         self._test('b[2]', 2)
         self._test('b[-1]', 3)
-        self.assertRaises(IndexError, self._test, 'b[4]', 1)
+        self.assertRaises(IndexError, set_value, self, 'b[4]', 1)
         self.assertEqual(self.locals.b, [1, 0, 2, 3])
 
     def set_sub_test(self):
@@ -56,7 +56,7 @@ class SetValueTest(unittest.TestCase):
         self._test('c.e[0]', 1)
         self._test('c.e[2]', 2)
         self._test('c.e[3].g', 3)
-        self.assertRaises(IndexError, self._test, 'c.e[4]', 1)
+        self.assertRaises(IndexError, set_value, self, 'c.e[4]', 1)
         self.assertEqual(self.locals.c.e, [1, 0, 2, f])
 
     def iter_backdoor_test(self):
