@@ -6,7 +6,7 @@
 #
 # Dummy Request objects
 #
-# $Id: request.py 6190 2004-10-19 12:57:36Z andrewm $
+# $Id: request.py 6618 2006-06-23 04:44:26Z andrewm $
 #
 
 import string, re
@@ -39,7 +39,7 @@ class DummyFieldRequest:
         self.__content_parts = []
 
     def has_field(self, field):
-        return self.fields.has_key(field)
+        return field in self.fields
 
     def field_value(self, field):
         return self.fields[field]
