@@ -6,7 +6,7 @@
 #
 # SessionServer functionality tests
 #
-# $Id: ses_server.py,v 1.9 2004/03/10 00:49:59 andrewm Exp $
+# $Id: ses_server.py 6562 2006-06-20 02:40:42Z andrewm $
 #
 
 import os, sys
@@ -74,3 +74,16 @@ class ServerCase(unittest.TestCase):
 
     def new_session(self):
         return Session()
+
+def have_server():
+    import socket
+    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    try:
+        try:
+            s.connect(('127.0.0.1', 34343))
+        except socket.error:
+            return False
+        else:
+            return True
+    finally:
+        s.close()
