@@ -6,7 +6,7 @@
 # 
 # Test if-elif-else flow constructs.
 #
-# $Id: control_flow.py 5987 2003-01-24 11:34:31Z andrewm $
+# $Id: control_flow.py 8851 2007-03-13 05:29:01Z andrewm $
 
 import unittest
 import albatross
@@ -29,7 +29,7 @@ class IfThenElifElseCase(albatross_test.AlbatrossTestCase):
         self.html_test("if-elif-else.html", -1, "negative\n")
 
     def check_unclosed(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "if-elif-fail.html", 0, "(none)")
+        self.test_raise("if-elif-fail.html")
 
 
 class ControlFlowSuite(unittest.TestSuite):
