@@ -6,7 +6,7 @@
 # 
 # Form functionality tests.
 #
-# $Id: form.py 6207 2005-04-19 09:53:58Z andrewm $
+# $Id: form.py 8871 2007-03-14 00:29:57Z andrewm $
 
 import unittest
 import albatross
@@ -76,24 +76,25 @@ class InputCase(albatross_test.AlbatrossTestCase):
 """)
 
     def check_missing_multiple(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "missing-multiple.html")
+        self.test_raise("missing-multiple.html")
 
     def check_illegal_multiple(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "illegal-multiple.html")
+        self.test_raise("illegal-multiple.html")
 
     def check_defined_single(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "defined-single.html")
+        self.test_raise("defined-single.html")
 
     def check_defined_multiple(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "defined-multiple.html")
+        self.test_raise("defined-multiple.html")
+
+    def check_multiple_mixed_implicit(self):
+        self.test_raise("multiple-mixed-implicit.html")
 
     def check_missing_name(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
-                          "missing-name.html")
+        self.test_raise("missing-name.html")
 
     def check_null_name(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
-                          "null-name.html")
+        self.test_raise("null-name.html")
 
 class InputSuite(unittest.TestSuite):
     test_list = (
@@ -112,6 +113,7 @@ class InputSuite(unittest.TestSuite):
         "check_illegal_multiple",
         "check_defined_single",
         "check_defined_multiple",
+        "check_multiple_mixed_implicit",
         "check_missing_name",
         "check_null_name",
     )
@@ -135,6 +137,32 @@ class SelectCase(albatross_test.AlbatrossTestCase):
         self.ctx.locals.sel = 'spam'
         self.html_test("select-value.html", '<select name="sel"><option value="spam" selected>canned meat product</option><option>eggs</option></select>')
 
+    def check_select_for(self):
+        self.ctx.locals.sel = 'eggs'
+        self.ctx.locals.values = 'spam', 'eggs'
+        self.html_test("select-for.html", '<select name="sel"><option>spam</option><option selected>eggs</option></select>')
+
+    def check_select_for_pair(self):
+        self.ctx.locals.sel = 'eggs'
+        self.ctx.locals.values = ('spam', 'canned meat product'), 'eggs'
+        self.html_test("select-for.html", '<select name="sel"><option value="spam">canned meat product</option><option selected>eggs</option></select>')
+
+    def check_select_for_label(self):
+        self.ctx.locals.sel = 'eggs'
+        self.ctx.locals.values = ('spam', 'canned meat product'), ('eggs', None)
+        self.html_test("select-for-label.html", '<select name="sel"><option value="spam">canned meat product</option><option selected>eggs</option></select>')
+
+    def check_select_for_content(self):
+        self.ctx.locals.sel = 'eggs'
+        self.ctx.locals.values = ('spam', 'canned meat product'), ('eggs', '')
+        self.html_test("select-for-content.html", '<select name="sel"><option value="spam">canned meat product</option><option selected>eggs</option></select>')
+
+    def check_select_select(self):
+        self.test_raise('select-select.html')
+
+    def check_bare_option(self):
+        self.test_raise('bare-option.html')
+
     def check_select_name_expr(self):
         self.ctx.locals.val = 'spam'
         self.html_test("select-name-expr.html", '<select name="sel"><option selected>spam</option><option>eggs</option></select>')
@@ -184,6 +212,12 @@ class SelectSuite(unittest.TestSuite):
         "check_select",
         "check_select_evaluate",
         "check_select_value",
+        "check_select_for",
+        "check_select_for_pair",
+        "check_select_for_label",
+        "check_select_for_content",
+        "check_select_select",
+        "check_bare_option",
         "check_select_name_expr",
         "check_select_optionexpr",
         "check_select_optionexpr_pairs",
@@ -223,8 +257,7 @@ class FormCase(albatross_test.AlbatrossTestCase):
     template_dir = "form"
 
     def check_nested_form(self):
-        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
-                          "nested-form.html")
+        self.test_raise("nested-form.html")
 
 
 class FormSuite(unittest.TestSuite):
