@@ -6,7 +6,7 @@
 # 
 # Include functionality tests.
 #
-# $Id: include.py 5987 2003-01-24 11:34:31Z andrewm $
+# $Id: include.py 8851 2007-03-13 05:29:01Z andrewm $
 
 import unittest
 import albatross
@@ -26,7 +26,7 @@ class IncludeCase(albatross_test.AlbatrossTestCase):
         self.html_test("nested.html", "plain\n")
 
     def check_nxfile(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "incnxfile.html", "(none)")
+        self.test_raise("incnxfile.html")
 
 class IncludeSuite(unittest.TestSuite):
     test_list = (
