@@ -6,7 +6,7 @@
 # 
 # Test macros.
 #
-# $Id: macro.py 6253 2006-01-10 04:56:40Z andrewm $
+# $Id: macro.py 8842 2007-03-12 06:04:09Z andrewm $
 
 import unittest
 import albatross
@@ -27,17 +27,37 @@ class MacroCase(albatross_test.AlbatrossTestCase):
     def check_macro_2arg(self):
         self.html_test("2arg.html", "xxx first second xxx\n")
 
+    def check_macro_simplearg(self):
+        self.html_test("simplearg.html", "xxx aaa bbb xxx\n")
+
+    def check_macro_argexpr(self):
+        self.ctx.locals.aaa = 'first'
+        self.ctx.locals.bbb = 'second'
+        self.html_test("argexpr.html", "xxx first second xxx\n")
+
     def check_macro_unnamed(self):
         self.html_test("unnamed.html", "xxx unnamed body xxx\n")
 
+    def check_macro_unnamedarg(self):
+        self.html_test("unnamedarg.html", "xxx argument")
+
+    def check_macro_nested(self):
+        self.html_test("nested.html", "aaa second bbb first ccc second ddd\n")
+
+    def check_macro_default(self):
+        self.html_test("default.html", "aaa first bbb\naaa catflap bbb\n")
+
+    def check_macro_defaultnested(self):
+        self.html_test("defaultnested.html", "aaa second bbb first ccc second ddd catflap eee\n")
+
     def check_macro_unclosed(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "unclosed.html", "(none)")
+        self.test_raise("unclosed.html")
 
     def check_macro_unclosedarg(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "unclosedarg.html", "(none)")
+        self.test_raise("unclosedarg.html")
 
     def check_macro_missingarg(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "missingarg.html", "(none)")
+        self.test_raise("missingarg.html")
 
 class MacroSuite(unittest.TestSuite):
     test_list = (
@@ -45,10 +65,16 @@ class MacroSuite(unittest.TestSuite):
         "check_macro_0arg_dup",
         "check_macro_1arg",
         "check_macro_2arg",
+        "check_macro_simplearg",
+        "check_macro_argexpr",
         "check_macro_unnamed",
+        "check_macro_nested",
+        "check_macro_default",
+        "check_macro_defaultnested",
         "check_macro_unclosed",
         "check_macro_unclosedarg",
         "check_macro_missingarg",
+        "check_macro_unnamedarg",
     )
     def __init__(self):
         unittest.TestSuite.__init__(self, map(MacroCase, self.test_list))
