@@ -6,7 +6,7 @@
 # 
 # Test "require" tag
 #
-# $Id: require.py 6204 2005-03-01 03:12:17Z andrewm $
+# $Id: require.py 8878 2007-03-14 05:38:02Z andrewm $
 
 import unittest
 import albatross
@@ -16,21 +16,18 @@ class RequireCase(albatross_test.AlbatrossTestCase):
     template_dir = "require"
 
     def check_require(self):
-        self.short_html_test("<al-require>", "")
-        self.assertRaises(albatross.ApplicationError, self.short_html_test,
-                          "<al-require nosuchattribute=\"\">", "")
+        self.test_str_html("<al-require>", "")
+        self.test_str_raise("<al-require nosuchattribute=\"\">")
 
     def check_require_version(self):
         fmt = "<al-require version=\"%d\">"
-        self.short_html_test(fmt % albatross.template_version, "")
-        self.short_html_test(fmt % (albatross.template_version - 1), "")
-        self.assertRaises(albatross.ApplicationError, self.short_html_test,
-                          fmt % (albatross.template_version + 1), "")
+        self.test_str_html(fmt % albatross.template_version, "")
+        self.test_str_html(fmt % (albatross.template_version - 1), "")
+        self.test_str_raise(fmt % (albatross.template_version + 1))
 
     def check_require_feature(self):
-        self.short_html_test("<al-require feature=\"\">", "")
-        self.assertRaises(albatross.ApplicationError, self.short_html_test,
-                          "<al-require feature=\"nosuchfeature\">", "")
+        self.test_str_html("<al-require feature=\"\">", "")
+        self.test_str_raise("<al-require feature=\"nosuchfeature\">")
 
 class RequireSuite(unittest.TestSuite):
     test_list = (
