@@ -6,7 +6,7 @@
 # 
 # Test whitespace.
 #
-# $Id: whitespace.py 5987 2003-01-24 11:34:31Z andrewm $
+# $Id: whitespace.py 8851 2007-03-13 05:29:01Z andrewm $
 
 import unittest
 import albatross
@@ -31,7 +31,7 @@ class WhitespaceCase(albatross_test.AlbatrossTestCase):
         self.html_test("all.html", "1\n       2\n    \t3")
 
     def check_bogus(self):
-        self.assertRaises(albatross.ApplicationError, self.html_test, "bogus.html", "(none)")
+        self.test_raise("bogus.html")
 
 class WhitespaceSuite(unittest.TestSuite):
     test_list = (
